# -*- coding: utf-8 -*-

# @File    : models.py
# @Date    : 2018-07-20
# @Author  : Peng Shiyu
# @program : 

# 保存到sqlite数据库

import os
from peewee import SqliteDatabase, Model, CharField, IntegerField, ForeignKeyField

DB_NAME = "china_area.sqlite"
BASE = os.path.dirname(os.path.abspath(__file__))
DB_PATH = os.path.join(BASE, DB_NAME)


DB = SqliteDatabase(DB_PATH)


class BaseModel(Model):
    class Meta:
        database = DB


class ChinaProvinceModel(BaseModel):
    name = CharField(default="")
    code = CharField(default="", unique=True)

    class Meta:
        db_table = 'china_province'


class ChinaCityModel(BaseModel):
    name = CharField(default="")
    code = CharField(default="", unique=True)
    province = ForeignKeyField(ChinaProvinceModel, field=ChinaProvinceModel.code, related_name='cities')

    class Meta:
        db_table = 'china_city'


class ChinaCountyModel(BaseModel):
    name = CharField(default="")
    code = CharField(default="", unique=True)
    city = ForeignKeyField(ChinaCityModel, field=ChinaCityModel.code, related_name='counties')

    class Meta:
        db_table = 'china_county'


# ======================
#  对数据库记录进行优化操作
# ======================

tables = [ChinaProvinceModel, ChinaCityModel, ChinaCountyModel]

for table in tables:
    if not table.table_exists():
        table.create_table()


def change_city():
    """
    将市级单位 直辖区 改为对应的 城市名称
    :return: None
    """
    cities = ChinaCityModel.filter(ChinaCityModel.name == "市辖区")
    for city in cities:
        print(city.province.name)


def change_city2():
    cities = ChinaCityModel.filter((ChinaCityModel.name == "省直辖县级行政区划") |
                                   (ChinaCityModel.name == "自治区直辖县级行政区划"))
    for city in cities:
        name = "{}{}".format(city.province.name, "直辖县")
        print(name)
        ChinaCityModel.update(name=name).where(ChinaCityModel.id == city.id).execute()


def _check_city(self):
    """
    检查城市名中重复名称
    :return: None
    """
    cities = ChinaCityModel.select()
    cities = [city.name for city in cities]
    print(len(cities))
    print(len(set(cities)))
    for city in set(cities):
        cities.remove(city)
    print(cities)


def _check_county(self):
    """
    检查县/区名中重复名称
    :return: None
    """
    counties = ChinaCountyModel.select()
    counties = [county.name for county in counties]
    print(len(counties))
    print(len(set(counties)))
    for county in set(counties):
        counties.remove(county)
    print(counties)


if __name__ == '__main__':
    # province = ChinaProvinceModel.filter(ChinaProvinceModel.name == "山西省").first()
    # for city in province.cities:
    #     print(city.name)
    change_city2()





# -*- coding: utf-8 -*-

# Define here the models for your scraped items
#
# See documentation in:
# https://doc.scrapy.org/en/latest/topics/items.html

import scrapy


class StatsSpiderItem(scrapy.Item):
    # define the fields for your item here like:
    model = scrapy.Field()



# -*- coding: utf-8 -*-

# Define your item pipelines here
#
# Don't forget to add your pipeline to the ITEM_PIPELINES setting
# See: https://doc.scrapy.org/en/latest/topics/item-pipeline.html

import logging
from models import ChinaAreaModel
from scrapy.exceptions import DropItem


class StatsSpiderPipeline(object):
    def process_item(self, item, spider):
        model = item["model"]

        ret = ChinaAreaModel.filter(ChinaAreaModel.code == model.code).first()

        logging.debug("*" * 40)
        for k, v in model.__data__.items():
            logging.debug(">>> {}: {}".format(k, v))

        if not ret:
            model.save()
            logging.debug("save successful name: %s" % model.name)
        else:
            logging.debug("save fail name: %s" % model.name)

        raise DropItem



import setuptools
import os
import requests


# 将markdown格式转换为rst格式
def md_to_rst(from_file, to_file):
    r = requests.post(url='http://c.docverter.com/convert',
                      data={'to':'rst','from':'markdown'},
                      files={'input_files[]':open(from_file,'rb')})
    if r.ok:
        with open(to_file, "wb") as f:
            f.write(r.content)


md_to_rst("README.md", "README.rst")

long_description = 'Add a fallback short description here'
if os.path.exists('README.rst'):
    long_description = open('README.rst', encoding="utf-8").read()

setuptools.setup(
    name="chinaarea",
    version="0.0.5",
    author="Peng Shiyu",
    license = 'MIT License',
    author_email="pengshiyuyx@gmail.com",
    description="get a china area by province, city or county",
    long_description=long_description,
    long_description_content_type="text/x-rst",
    url="https://github.com/mouday/chinaarea",
    packages=setuptools.find_packages(),
    classifiers=(
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ),
    install_requires = ["peewee==3.3.4", "jieba==0.39"],        # 常用
    package_data = {
            # If any package contains *.txt or *.rst files, include them:
            'chinaarea': ['stats_spider/*.sqlite']
    },
)

"""
打包上传指令：
python setup.py sdist bdist_wheel

twine upload dist/*

tree /F > tree.txt
"""


# -*- coding: utf-8 -*-

# @File    : chinaarea.py
# @Date    : 2018-07-21
# @Author  : Peng Shiyu

"""
#################### 说明 #########################
一个省 -> 多个市，一个市 -> 多个县
一个县 -> 一个市， 一个市 -> 一个省

备注：县级名称单位可能会重复， 返回第一个查到的市级单位
来源：国家统计局 2017年统计用区划代码和城乡划分代码(截止2017年10月31日)
地址：http://www.stats.gov.cn/tjsj/tjbz/tjyqhdmhcxhfdm/2017/index.html
"""
import logging
import os
import sys

import jieba

BASE = os.path.dirname(os.path.abspath(__file__))
sys.path.insert(0, BASE)
from stats_spider.models import ChinaProvinceModel, ChinaCityModel, ChinaCountyModel

__all__ = ["ChinaArea"]


class ChinaArea(object):
    """
    获取中国行政区域划分
    """
    def get_provinces(self, province_name=None):
        """
        获取中国所有省份名称
        :return: {list} 省份名称
        """
        if province_name is None:
            provinces = ChinaProvinceModel.select()
        else:
            provinces = ChinaProvinceModel.filter(ChinaProvinceModel.name.contains(province_name))
        return [province.name for province in provinces]

    def get_provinces_by_city(self, city_name):
        """
        通过城市名获取所属省份
        :param city_name: {str} 城市名称
        :return: {list} 省份名称
        """
        cities = ChinaCityModel.filter(ChinaCityModel.name.contains(city_name))
        return [city.province.name for city in cities]

    def get_cities(self, city_name=None):
        """
        获取所有城市名称
        :return: {list} 城市名称
        """
        if city_name is None:
            cities = ChinaCityModel.select()
        else:
            cities = ChinaCityModel.filter(ChinaCityModel.name.contains(city_name))
        return [city.name for city in cities]

    def get_cities_by_province(self, province_name):
        """
        通过省份名称获取下面的所有城市
        :param province_name: {str} 省份名称
        :return: {list} 城市名称
        """
        provinces = ChinaProvinceModel.filter(ChinaProvinceModel.name.contains(province_name))

        city_list = []

        for province in provinces:
            for city in province.cities:
                city_list.append(city.name)

        return city_list

    def get_cities_by_county(self, county_name):
        """
        通过县/区名获取所属城市
        :param county_name: {str} 县/区名称
        :return: {list} 城市名称
        """
        counties = ChinaCountyModel.filter(ChinaCountyModel.name.contains(county_name))
        return [county.city.name for county in counties]

    def get_counties(self, county_name=None):
        """
        获取的所有县/区
        :return: {list} 县/区名称
        """
        if county_name is None:
            counties = ChinaCountyModel.select()
        else:
            counties = ChinaCountyModel.filter(ChinaCountyModel.name.contains(county_name))
        return [county.name for county in counties]

    def get_counties_by_city(self, city_name):
        """
        通过城市名称获取下面的所有县/区
        :param city_name: {str} 城市名称
        :return: {list} 县/区名称
        """
        cities = ChinaCityModel.filter(ChinaCityModel.name.contains(city_name))

        county_list = []
        for city in cities:
            for county in city.counties:
                county_list.append(county.name)

        return county_list

    def is_province(self, address):
        """
        判断字符串是否是一个省名
        :param address: {str} 字符串
        :return: {bool} 是省名 True, 不是省名 False
        """
        province = ChinaProvinceModel.filter(ChinaProvinceModel.name == address).first()
        return True if province else False

    def is_city(self, address):
        """
        判断字符串是否是一个城市名
        :param address: {str} 字符串
        :return: {bool} 是城市名 True, 不是城市名 False
        """
        city = ChinaCityModel.filter(ChinaCityModel.name == address).first()
        return True if city else False

    def is_county(self, address):
        """
        判断字符串是否是一个县/区名
        :param address: {str} 字符串
        :return: {bool} 是县/区名 True, 不是县/区名 False
        """
        county = ChinaCountyModel.filter(ChinaCountyModel.name == address).first()
        return True if county else False

    def find_areas(self, address):
        """
        查找语句中的省份，地市，县/区名称
        :param address: {str} 字符串
        :return: {dict}
            {
                "provinces": set(),  省
                "cities": set(),     市
                "counties": set()    县/区
            }
        """
        jieba.setLogLevel(logging.INFO)
        words = jieba.lcut(address)
        areas = {
            "provinces": set(),
            "cities": set(),
            "counties": set()
        }
        for word in words:
            if len(word) < 2:
                continue
            provinces = self.get_provinces(word)
            if provinces:
                areas["provinces"].update(provinces)
            cities = self.get_cities(word)
            if cities:
                areas["cities"].update(cities)
            counties = self.get_counties(word)
            if counties:
                areas["counties"].update(counties)

        return areas


if __name__ == '__main__':
    ca = ChinaArea()
    company = "武汉市蔡甸区战略性新兴产业发展引导基金"
    ret = ca.find_areas(company)
    for k, v in ret.items():
        print("{}: {}".format(k, v))



# -*- coding: utf-8 -*-

# @File    : stats_spider1.py
# @Date    : 2018-07-20
# @Author  : Peng Shiyu
# @program : 中国 省 市 县 行政区域爬虫

"""
来源：国家统计局 2017年统计用区划代码和城乡划分代码(截止2017年10月31日)
地址：http://www.stats.gov.cn/tjsj/tjbz/tjyqhdmhcxhfdm/2017/index.html
"""
import os
import sys

import re

from china_area.stats_spider.models import ChinaProvinceModel, ChinaCityModel, ChinaCountyModel

BASE = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))
sys.path.insert(0, BASE)

from scrapy import Spider, Selector, Request
from items import StatsSpiderItem
from models import ChinaAreaModel


class AreaSpider(Spider):
    """
    解析出省份列表
    """
    name = 'area_spider'
    # download_delay = 1
    allowed_domains = ['stats.gov.cn']
    start_urls = ['http://www.stats.gov.cn/tjsj/tjbz/tjyqhdmhcxhfdm/2017/index.html']

    def parse(self, response):
        provinces = response.css(".provincetr td")
        for province in provinces:
            href = province.css("a::attr(href)").extract_first("")
            name = province.css("a::text").extract_first("")
            url = response.urljoin(href)
            if href == "":
                continue

            code = re.findall("\d+", href)[0]

            model = ChinaProvinceModel()
            model.name = name
            model.code = code
            try:
                model.save()
                self.log("save success name: %s" % name)
            except Exception as e:
                self.log("save fail : %s" % e)

            yield Request(url, meta={"province": model}, callback=self.parse_city)

    def parse_city(self, response):
        province = response.meta.get("province")

        # 城市
        citys = response.css(".citytr")
        for city in citys:
            code = city.xpath("./td[1]").xpath("string(.)").extract_first("")
            name = city.xpath("./td[2]").xpath("string(.)").extract_first("")
            href = city.xpath("./td[2]/a/@href").extract_first("")

            url = response.urljoin(href)

            model = ChinaCityModel()
            model.name = name
            model.code = code
            model.province = province
            try:
                model.save()
                self.log("save success name: %s" % name)
            except Exception as e:
                self.log("save fail : %s" % e)

            yield Request(url, meta={"city": model}, callback=self.parse_county)

    def parse_county(self, response):
        city = response.meta.get("city")

        # 县
        counties = response.css(".countytr")
        for county in counties:
            code = county.xpath("./td[1]").xpath("string(.)").extract_first("")
            name = county.xpath("./td[2]").xpath("string(.)").extract_first("")

            model = ChinaCountyModel()
            model.name = name
            model.code = code
            model.city = city

            try:
                model.save()
                self.log("save success name: %s" % name)
            except Exception as e:
                self.log("save fail : %s" % e)


if __name__ == '__main__':
    from scrapy import cmdline
    cmdline.execute("scrapy crawl area_spider".split())

